using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration.Install;
using System.IO;
using System.Collections.Specialized;

/* 
   | Copyright 2007                |
   | Benjamin Day                  |
   | Benjamin Day Consulting, Inc. |
   | http://www.benday.com         |
   | http://blog.benday.com        |
    */

namespace Com.Benday.InstallerActions
{
	public class EditConfigurationFilesAction
	{
		public EditConfigurationFilesAction()
		{
		
		}

		public void Install(StringDictionary parameters)
		{
			string targetDir = parameters["targetDirectory"];

			if (Directory.Exists(targetDir) == false)
			{
				throw new DirectoryNotFoundException(String.Format("Could not find the target directory ({0}).", targetDir));
			}

			string webConfigDatabaseKeys = parameters["webConfigDatabaseKeys"];
			string webConfigUpdatePathToFilenamesKey = parameters["webConfigUpdatePathToFilenamesKey"];
			string enterpriseLibraryDbConfigFile = parameters["enterpriseLibraryDbConfigFile"];
			string hibernateConfigFile = parameters["hibernateConfigFile"];
			string webConfigConnectionString = parameters["webConfigConnectionString"];

			string webConfigPath = Path.Combine(targetDir, "web.config");

			string hibernateConfigPath = null;
			if (hibernateConfigFile != null)
			{
				hibernateConfigPath = Path.Combine(targetDir, hibernateConfigFile);
			}

			string enterpriseLibraryDataConfigPath = null;
			if (enterpriseLibraryDbConfigFile != null)
			{
				enterpriseLibraryDataConfigPath = Path.Combine(targetDir, enterpriseLibraryDbConfigFile);
			}

			UpdatePathToFilenamesReferencedInWebConfig(targetDir, webConfigPath, webConfigUpdatePathToFilenamesKey);

			bool doWebConfigDatabaseKeys = false;
			bool doWebConfigConnectionString = false;
			bool doEnterpriseLibraryDatabaseConfig = false;
			bool doHibernateConfig = false;

			if (webConfigDatabaseKeys != null && File.Exists(webConfigPath) == false)
			{
				throw new FileNotFoundException(String.Format("Web.config did not exist at {0}.", webConfigPath));
			}
			else if (webConfigDatabaseKeys != null)
			{
				doWebConfigDatabaseKeys = true;
			}

			if (webConfigConnectionString != null && File.Exists(webConfigPath) == false)
			{
				throw new FileNotFoundException(String.Format("Web.config did not exist at {0}.", webConfigPath));
			}
			else if (webConfigConnectionString != null)
			{
				doWebConfigConnectionString = true;
			}

			if (enterpriseLibraryDbConfigFile != null && File.Exists(enterpriseLibraryDataConfigPath) == false)
			{
				throw new FileNotFoundException(String.Format("Database configuration file did not exist at {0}.", 
					enterpriseLibraryDataConfigPath));
			}
			else if (enterpriseLibraryDbConfigFile != null)
			{
				doEnterpriseLibraryDatabaseConfig = true;
			}

			if (hibernateConfigFile != null && File.Exists(hibernateConfigPath) == false)
			{
				throw new FileNotFoundException(String.Format("Hibernate configuration file did not exist at {0}.", 
					hibernateConfigPath));
			}
			else if (hibernateConfigFile != null)
			{
				doHibernateConfig = true;
			}

			try
			{
				if (doWebConfigDatabaseKeys == true || doEnterpriseLibraryDatabaseConfig == true || doHibernateConfig == true)
				{
					bool isFirstConfiguration = true;

					using (SqlConnectionEditor editor = new SqlConnectionEditor())
					{
						editor.ApplyToAll = true;

						if (doEnterpriseLibraryDatabaseConfig == true)
						{
							DatabaseConfigurationParser parser = new DatabaseConfigurationParser(enterpriseLibraryDataConfigPath);

							if (isFirstConfiguration == true || editor.ApplyToAll == false)
							{
								editor.ConnectionBuilder.ConnectionString = parser.ConnectionString;
								isFirstConfiguration = false;
								editor.Text = "Enterprise Library Database Configuration";
								editor.ShowDialog();
							}

							parser.UserName = editor.ConnectionBuilder.Username;
							parser.Password = editor.ConnectionBuilder.Password;
							parser.Database = editor.ConnectionBuilder.Database;
							parser.Server = editor.ConnectionBuilder.Server;
							parser.UseTrustedConnection = editor.ConnectionBuilder.IsUsingIntegratedSecurity;

							parser.Save();
						}

						if (doHibernateConfig == true)
						{
							HibernateDatabaseConfigurationParser parser =
								new HibernateDatabaseConfigurationParser(hibernateConfigPath);

							if (isFirstConfiguration == true || editor.ApplyToAll == false)
							{
								editor.ConnectionBuilder.ConnectionString = parser.ConnectionString;
								isFirstConfiguration = false;
								editor.Text = "NHibernate Database Configuration";
								editor.ShowDialog();
							}

							parser.UserName = editor.ConnectionBuilder.Username;
							parser.Password = editor.ConnectionBuilder.Password;
							parser.Database = editor.ConnectionBuilder.Database;
							parser.Server = editor.ConnectionBuilder.Server;
							parser.UseTrustedConnection = editor.ConnectionBuilder.IsUsingIntegratedSecurity;

							parser.Save();
						}

						if (doWebConfigDatabaseKeys == true)
						{
							string[] keys = webConfigDatabaseKeys.Split(';');

							if (keys.Length > 0)
							{
								WebConfigParser parser =
									new WebConfigParser(webConfigPath);

								if (isFirstConfiguration == true || editor.ApplyToAll == false)
								{
									editor.ConnectionBuilder.ConnectionString = parser[keys[0]];
									editor.Text = "Web.config Database Configurations";
									editor.ShowDialog();
									isFirstConfiguration = false;
								}

								foreach (string key in keys)
								{
									parser[key] = editor.ConnectionBuilder.ConnectionString;
								}

								parser.Save();
							}
						}

						if (doWebConfigConnectionString == true)
						{
							WebConfigConnectionStringParser parser =
								new WebConfigConnectionStringParser(webConfigPath, webConfigConnectionString);

							if (isFirstConfiguration == true || editor.ApplyToAll == false)
							{
								editor.ConnectionBuilder.ConnectionString = parser.ConnectionString;
								editor.Text = "Web.config Database Connection String";
								editor.ShowDialog();
								isFirstConfiguration = false;
							}

							parser.UserName = editor.ConnectionBuilder.Username;
							parser.Password = editor.ConnectionBuilder.Password;
							parser.Database = editor.ConnectionBuilder.Database;
							parser.Server = editor.ConnectionBuilder.Server;
							parser.UseTrustedConnection = editor.ConnectionBuilder.IsUsingIntegratedSecurity;
							
							parser.Save();
						}
					}
				}
			}
			catch (Exception ex)
			{
				throw new Exception(ex.ToString());				
			}

		}

		private void UpdatePathToFilenamesReferencedInWebConfig(
			string targetDir, string webConfigPath, string webConfigUpdatePathToFilenamesKeys)
		{
			string[] keys = webConfigUpdatePathToFilenamesKeys.Split(';');

			if (keys.Length > 0 && (keys.Length % 2) != 0)
			{
				throw new InvalidOperationException("Update path to filenames value should have an even number of items.");
			}
			else 
			{
				WebConfigParser parser =
					new WebConfigParser(webConfigPath);

				string key = null;
				string val = null;

				for (int index = 0; index < keys.Length; index+=2)
                {
					key = keys[index];
					val = keys[index + 1];

					val = Path.Combine(targetDir, val);

					parser[key] = val;
                }

				parser.Save();
			}
		}
	}
}